using System;
using System.Collections.Generic;
using DataContracts;
namespace Ato.EN.IntegrationServices.CodeGenerationTRT {
    public partial class TRT2020Validator {
    
    #region Cross Form Rules - not supported yet

        #region VR.ATO.GEN.430008

        /*  VR.ATO.GEN.430008
                    An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "IEE") > 0) AND RP:entity.identifier.TFN = (ANY OCCURRENCE OF([IEE50])) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE50] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)

            Technical Business Rule Format:
                    (CountDocument('IEE') > 0) AND AnyOccurrence(^IEE50, ^IEE50 = ^TFN)

        Data Elements:
    
            ^IEE50 = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
    
            ^TFN = RP
            */
        protected void VRATOGEN430008(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.430008
            
            assertion = false; // = (@"IEE") > 0 && ( == null ? false : .Any(IEE50Repeat =>  == report.RPIdentifierTFN)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430008", Severity = ProcessMessageSeverity.Error,
                    Description = @"The Family Trust TFN on the IEE must not be the same as the main entity TFN",
                    LongDescription = @"An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430008"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.430008

        #region VR.ATO.GEN.438002

        /*  VR.ATO.GEN.438002
                    If the response to the question to whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust (Attributed foreign income) is 'Yes, an International Dealings Schedule must be attached

                Legacy Rule Format:
                    IF (bafpr3.xx.xx:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator = TRUE) AND (COUNT(SCHEDULE = "IDS") = 0) 
                        RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^TRT138 = TRUE) AND (CountDocument('IDS') = 0)

        Data Elements:
    
            ^TRT138 = RP.JFOREIGN:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
            */
        protected void VRATOGEN438002(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438002
            
            assertion = false; // = (report.TRT138 == true && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438002", Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not attached",
                    LongDescription = @"The answer to the question 'Did the entity have either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust?' is 'Yes', and no International Dealings Schedule is attached.",
                    //Location = "/xbrli:xbrl/tns:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438002"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT138", Value = GetValueOrEmpty(report.TRT138) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438002

        #region VR.ATO.GEN.438017

        /*  VR.ATO.GEN.438017
                    When you were affected by the Thin capitalisation provisions, the International Dealings Schedule (IDS) must be attached

                Legacy Rule Format:
                    IF (RP:bafpo6.xx.xx:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator = TRUE) AND (COUNT(SCHEDULE = "IDS") = 0)
                      RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    (^TRT449 = TRUE) AND (CountDocument('IDS') = 0)

        Data Elements:
    
            ^TRT449 = RP:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
            */
        protected void VRATOGEN438017(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438017
            
            assertion = false; // = (report.TRT449 == true && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438017", Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"If the thin capitalisation rules apply  then International Dealings Schedule must be attached",
                    //Location = "/xbrli:xbrl/tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438017"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT449", Value = GetValueOrEmpty(report.TRT449) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438017

        #region VR.ATO.GEN.438022

        /*  VR.ATO.GEN.438022
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "A - Small business active asset reduction") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS127] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS127] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'A - Small business active asset reduction') AND (CountDocument('CGTS') = 1) AND ^CGTS127 = NULL

        Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS127 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
            */
        protected void VRATOGEN438022(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438022
            
            assertion = false; // = (TRT448 == @"A - Small business active asset reduction" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438022", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business active asset reduction amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438022"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = GetValueOrEmpty(TRT448) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438022

        #region VR.ATO.GEN.438023

        /*  VR.ATO.GEN.438023
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business retirement exemption (Subdivision152-D)", then Small business retirement exemption amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "B - Small business retirement exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS128] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS128] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'B - Small business retirement exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS128 = NULL

        Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS128 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount
            */
        protected void VRATOGEN438023(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438023
            
            assertion = false; // = (TRT448 == @"B - Small business retirement exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business retirement exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business retirement exemption (Subdivision152-D)"", then Small business retirement exemption amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438023"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = GetValueOrEmpty(TRT448) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438023

        #region VR.ATO.GEN.438024

        /*  VR.ATO.GEN.438024
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "C - Small business roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS129] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS129] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'C - Small business roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS129 = NULL

        Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS129 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
            */
        protected void VRATOGEN438024(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438024
            
            assertion = false; // = (TRT448 == @"C - Small business roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438024", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438024"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = GetValueOrEmpty(TRT448) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438024

        #region VR.ATO.GEN.438025

        /*  VR.ATO.GEN.438025
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "D - Small business 15 year exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS138] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS138] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'D - Small business 15 year exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS138 = NULL

        Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS138 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
            */
        protected void VRATOGEN438025(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438025
            
            assertion = false; // = (TRT448 == @"D - Small business 15 year exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438025", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business 15 year exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438025"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = GetValueOrEmpty(TRT448) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438025

        #region VR.ATO.GEN.438026

        /*  VR.ATO.GEN.438026
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Foreign resident CGT exemption (Division 855)", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "E - Foreign resident CGT exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS140] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS140] = CGTS:RP:rvctc3.02.11:Capital.Gains.ForeignResidentExemption.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'E - Foreign resident CGT exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS140 = NULL

        Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS140 = CGTS:RP:rvctc3.02.11:Capital.Gains.ForeignResidentExemption.Amount
            */
        protected void VRATOGEN438026(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438026
            
            assertion = false; // = (TRT448 == @"E - Foreign resident CGT exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438026", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a foreign resident amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Foreign resident CGT exemption (Division 855)"", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438026"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = GetValueOrEmpty(TRT448) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438026

        #region VR.ATO.GEN.438027

        /*  VR.ATO.GEN.438027
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "F - Scrip for scrip roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS45] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS45] = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'F - Scrip for scrip roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS45 = NULL

        Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS45 = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
            */
        protected void VRATOGEN438027(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438027
            
            assertion = false; // = (TRT448 == @"F - Scrip for scrip roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438027", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438027"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = GetValueOrEmpty(TRT448) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438027

        #region VR.ATO.GEN.438028

        /*  VR.ATO.GEN.438028
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Demerger exemption (Subdivision 125-C)", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed.

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "H - Demerger exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS47] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS47] = CGTS:RP:rvctc3.02.11:Capital.Gains.DemergingEntityExemption.Amount

            Technical Business Rule Format:
                    (^TRT448 = 'H - Demerger exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS47 = NULL

        Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^CGTS47 = CGTS:RP:rvctc3.02.11:Capital.Gains.DemergingEntityExemption.Amount
            */
        protected void VRATOGEN438028(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438028
            
            assertion = false; // = (TRT448 == @"H - Demerger exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438028", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded by a demerging entity amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Demerger exemption (Subdivision 125-C)"", then Capital gains disregarded by a demerging entity amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438028"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = GetValueOrEmpty(TRT448) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438028

        #region VR.ATO.TRT.432011

        /*  VR.ATO.TRT.432011
                    The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.

                Legacy Rule Format:
                    IF [TRT1] <> NULL AND (COUNT(SCHEDULE = "IEE") > 0) AND (ANY OCCURRENCE OF ([IEE116] = "R") AND ([IEE82] <> [TRT1]))
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IEE82] = IEE:RP:rvctc3.02.00:Elections.Revocation.Year
                    [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
                    [TRT1] = TRT:RP:pyin.02.37:Report.TargetFinancial.Year

            Technical Business Rule Format:
                    ^TRT1 <> NULL AND (CountDocument('IEE') > 0) AND ((AnyOccurrence(^IEE116, ^IEE116 = 'R')) AND (^IEE82 <> ^TRT1))

        Data Elements:
    
            ^TRT1 = RP:Report.TargetFinancial.Year
    
            ^IEE82 = IEE:RP:rvctc3.02.00:Elections.Revocation.Year
    
            ^IEE116 = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
            */
        protected void VRATOTRT432011(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432011
            
            assertion = false; // = (report.TRT1 != null && @"IEE") > 0 && (( == null ? false : .Any(IEE116Repeat =>  == @"R")) &&  != report.TRT1.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430176", Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of revocation must be the same as the year of the return",
                    LongDescription = @"The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.",
                    //Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432011"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT1", Value = GetValueOrEmpty(report.TRT1) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432011

        #region VR.ATO.TRT.432017

        /*  VR.ATO.TRT.432017
                    Family Trust Election status code selected requires a Family trust election, revocation or variation schedule to be completed and attached to return and there is no schedule present.

                Legacy Rule Format:
                    IF [TRT29] = SET("R","V") AND  (COUNT(SCHEDULE = "FTER") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT29] = TRT:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
                    InSet(^TRT29, '"R","V"') AND  (CountDocument('FTER') = 0)

        Data Elements:
    
            ^TRT29 = RP:Elections.FamilyTrustElectionRevocation.Code
            */
        protected void VRATOTRT432017(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432017
            
            assertion = false; // = (IsMatch(report.TRT29, @"^(R|V)$") && @"FTER") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432017", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family trust election or revocation schedule must be present",
                    //Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432017"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT29", Value = GetValueOrEmpty(report.TRT29) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432017

        #region VR.ATO.TRT.432019

        /*  VR.ATO.TRT.432019
                    An IEE schedule is attached; however the IEE specified income year has not been completed on the main form.

                Legacy Rule Format:
                    IF [TRT30] = NULL AND (COUNT(SCHEDULE = "IEE") > 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT30] = TRT:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

            Technical Business Rule Format:
                    ^TRT30 = NULL AND (CountDocument('IEE') > 0)

        Data Elements:
    
            ^TRT30 = RP:Elections.InterposedEntityElectionStatus.Year
            */
        protected void VRATOTRT432019(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432019
            
            assertion = false; // = (report.TRT30 == null && @"IEE") > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430178", Severity = ProcessMessageSeverity.Error,
                    Description = @"IEE specified income year must be present",
                    LongDescription = @"If an IEE schedule is present, the Interposed entity election status - Specified income year must be present",
                    //Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432019"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT30", Value = GetValueOrEmpty(report.TRT30) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432019

        #region VR.ATO.TRT.432020

        /*  VR.ATO.TRT.432020
                    The Interposed Entity Election code indicates a revocation is being made, however no schedule is attached.

                Legacy Rule Format:
                    IF ([TRT31] = "R") AND (COUNT(SCHEDULE = "IEE") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT31] = TRT:RP:rvctc3.02.00:Elections.InterposedEntityElectionRevocation.Code

            Technical Business Rule Format:
                    (^TRT31 = 'R') AND (CountDocument('IEE') = 0)

        Data Elements:
    
            ^TRT31 = RP:Elections.InterposedEntityElectionRevocation.Code
            */
        protected void VRATOTRT432020(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432020
            
            assertion = false; // = (report.TRT31 == @"R" && @"IEE") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430180", Severity = ProcessMessageSeverity.Error,
                    Description = @"If the Interposed Entity Election revocation code is present, an IEE schedule must be present",
                    //Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432020"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT31", Value = GetValueOrEmpty(report.TRT31) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432020

        #region VR.ATO.TRT.432042

        /*  VR.ATO.TRT.432042
                    If Net capital gain is greater than $10,000, a Capital Gains Tax schedule must be present

                Legacy Rule Format:
                    IF [TRT133] > 10000 AND [TRT44] <> TRUE AND  (COUNT(SCHEDULE = "CGTS") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT44] = TRT:RP:pyde.02.00:OrganisationDetails.TaxConsolidationStatus.Indicator
                    [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    ^TRT133 > 10000 AND ^TRT44 <> TRUE AND  (CountDocument('CGTS') = 0)

        Data Elements:
    
            ^TRT44 = RP:OrganisationDetails.TaxConsolidationStatus.Indicator
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
            */
        protected void VRATOTRT432042(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432042
            
            assertion = false; // = (report.TRT133.GetValueOrDefault() > 10000 && report.TRT44 != true && @"CGTS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432042", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital Gains Tax schedule must be present",
                    LongDescription = @"If Net capital gain is greater than $10,000, a Capital Gains Tax schedule must be present",
                    //Location = "/xbrli:xbrl/tns:OrganisationDetails.TaxConsolidationStatus.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432042"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT44", Value = GetValueOrEmpty(report.TRT44) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432042

        #region VR.ATO.TRT.432043

        /*  VR.ATO.TRT.432043
                    Payment Summary schedule must be present if Gross payments where ABN not quoted is present

                Legacy Rule Format:
                    IF ([TRT46] + [TRT47]) > 0 AND (COUNT(SCHEDULE = "PSS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    (^TRT46 + ^TRT47) > 0 AND (CountDocument('PSS') = 0)

        Data Elements:
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
        protected void VRATOTRT432043(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432043
            
            assertion = false; // = (report.TRT46.GetValueOrDefault() + report.TRT47.GetValueOrDefault() > 0 && @"PSS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430019", Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule must be present if Gross payments where ABN not quoted is present",
                    //Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432043"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432043

        #region VR.ATO.TRT.432044

        /*  VR.ATO.TRT.432044
                    The sum of gross payment with indicator 'N' in the payment summary schedule must equal the sum of total gross payment where ABN not quoted for primary production AND total gross payment where ABN not quoted for non-primary production on the main return form

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") > 0) AND SUM([PSS19]) <> [TRT46] + [TRT47] 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS19] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') > 0) AND Sum(^PSS19) <> ^TRT46 + ^TRT47

        Data Elements:
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^PSS19 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
        protected void VRATOTRT432044(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432044
            
            assertion = false; // = (@"PSS") > 0 && ( == null ? 0 : .Sum(f => f.)) != report.TRT46.GetValueOrDefault() + report.TRT47.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430020", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment where ABN not quoted must equal sum of payments on schedule",
                    LongDescription = @"The sum of gross payment with indicator 'N' in the Payment Summary schedule must equal the sum of Primary production and Non-primary production Gross payments where ABN not quoted on main return form",
                    //Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432044"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432044

        #region VR.ATO.TRT.432051

        /*  VR.ATO.TRT.432051
                    The sum of gross payment with indicator 'F' in the Payment Summary schedule must equal total gross payment subject to foreign resident withholding (excluding capital gains) non-primary production on the main return form.

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "PSS") > 0) AND ([TRT49] <> SUM([PSS18]))
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS18] = PSS:RP.Payer.{PSSeqNum}:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
                    [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') > 0) AND (^TRT49 <> Sum(^PSS18))

        Data Elements:
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^PSS18 = PSS:RP.Payer.{PSSeqNum}:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            */
        protected void VRATOTRT432051(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432051
            
            assertion = false; // = (@"PSS") > 0 && report.TRT49.GetValueOrDefault() != ( == null ? 0 : .Sum(f => f.)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432749", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment - Foreign Resident Withholding (excluding capital gains) is incorrect",
                    LongDescription = @"The sum of gross payments with indicator 'F' in the Payment Summary schedule must be equal to total gross payment subject to foreign resident withholding (excluding capital gains) non-primary production on the main return form.",
                    //Location = "/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432051"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432051

        #region VR.ATO.TRT.432088

        /*  VR.ATO.TRT.432088
                    If an amount greater than zero has been entered at Tax withheld from payments where ABN not quoted, then a Payment Summary schedule must be present AND it must include the same total amount of Tax withheld from payments where ABN not quoted

                Legacy Rule Format:
                    IF ([TRT73] > 0) AND ((COUNT(SCHEDULE = "PSS") = 0) OR (SUM([PSS23]) <> [TRT73]))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [PSS23] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                    [TRT73] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
                    (^TRT73 > 0) AND ((CountDocument('PSS') = 0) OR (Sum(^PSS23) <> ^TRT73))

        Data Elements:
    
            ^TRT73 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^PSS23 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
            */
        protected void VRATOTRT432088(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432088
            
            assertion = false; // = (report.TRT73.GetValueOrDefault() > 0 && (@"PSS") == 0 || ( == null ? 0 : .Sum(f => f.)) != report.TRT73.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430183", Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule must include Tax withheld where ABN not quoted amount",
                    LongDescription = @"If an amount greater than zero has been entered at Tax withheld from payments where ABN no quoted, then a Payment Summary schedule must be present and it must include the same total amount of Tax withheld from payments where ABN no quoted",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432088"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT73", Value = GetValueOrEmpty(report.TRT73) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432088

        #region VR.ATO.TRT.432115

        /*  VR.ATO.TRT.432115
                    A rental schedule is attached AND rental income AND deduction amounts are not present in the return form.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND [TRT89] = NULL AND ([TRT90] = 0 OR [TRT90] = NULL) AND ([TRT91] = 0 OR [TRT91] = NULL) AND ([TRT92] = 0 OR [TRT92] = NULL) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND ^TRT89 = NULL AND (^TRT90 = 0 OR ^TRT90 = NULL) AND (^TRT91 = 0 OR ^TRT91 = NULL) AND (^TRT92 = 0 OR ^TRT92 = NULL)

        Data Elements:
    
            ^TRT89 = RP.JAUS:Income.Operating.RentalIncomeGross.Amount
    
            ^TRT90 = RP.JAUS:Expense.Interest.Amount
    
            ^TRT91 = RP.JAUS:Expense.CapitalWorksDeduction.Amount
    
            ^TRT92 = RP.JAUS:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
        protected void VRATOTRT432115(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432115
            
            assertion = false; // = (@"RS") > 0 && report.TRT89 == null && (report.TRT90 == 0 || report.TRT90 == null) && (report.TRT91 == 0 || report.TRT91 == null) && (report.TRT92 == 0 || report.TRT92 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430082", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent and Rental deduction amounts must be present",
                    LongDescription = @"If a Rental property schedule is present, Gross rent or rental deductions must be present",
                    //Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432115"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT89", Value = GetValueOrEmpty(report.TRT89) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT90", Value = GetValueOrEmpty(report.TRT90) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT91", Value = GetValueOrEmpty(report.TRT91) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT92", Value = GetValueOrEmpty(report.TRT92) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432115

        #region VR.ATO.TRT.432116

        /*  VR.ATO.TRT.432116
                    Total gross rent of schedules AND gross rent value in return must be equal

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND (SUM(ALL OCCURRENCES OF([RS38] + [RS39]))) <> [TRT89]
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS38] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.Rental.Amount
                    [RS39] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalRelatedOther.Amount
                    [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (Sum((^RS38 + ^RS39))) <> ^TRT89

        Data Elements:
    
            ^TRT89 = RP.JAUS:Income.Operating.RentalIncomeGross.Amount
    
            ^RS38 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.Rental.Amount
    
            ^RS39 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalRelatedOther.Amount
            */
        protected void VRATOTRT432116(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432116
            
            assertion = false; // = (@"RS") > 0 &&  + ) != report.TRT89.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430083", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent must equal the total Gross rent in the Rental Schedules",
                    LongDescription = @"The Gross rent amount in the main form must equal the total Gross rent present in all Rental property schedules",
                    //Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432116"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT89", Value = GetValueOrEmpty(report.TRT89) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432116

        #region VR.ATO.TRT.432117

        /*  VR.ATO.TRT.432117
                    Total net rent of schedule AND net rent value in return must be equal

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND SUM([RS59]) <> ([TRT89] - [TRT90] - [TRT91] - [TRT92]) 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS59] = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalNet.Amount
                    [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND Sum(^RS59) <> (^TRT89 - ^TRT90 - ^TRT91 - ^TRT92)

        Data Elements:
    
            ^TRT89 = RP.JAUS:Income.Operating.RentalIncomeGross.Amount
    
            ^RS59 = RS:RP:bafpr2.02.04:Income.RealEstateProperty.RentalNet.Amount
    
            ^TRT90 = RP.JAUS:Expense.Interest.Amount
    
            ^TRT91 = RP.JAUS:Expense.CapitalWorksDeduction.Amount
    
            ^TRT92 = RP.JAUS:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
        protected void VRATOTRT432117(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432117
            
            assertion = false; // = (@"RS") > 0 && ( == null ? 0 : .Sum(f => f.)) != report.TRT89.GetValueOrDefault() - report.TRT90.GetValueOrDefault() - report.TRT91.GetValueOrDefault() - report.TRT92.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430084", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net rent must equal the total net rent in the Rental Schedules",
                    LongDescription = @"The net rent amount in the main form (Gross rent minus rental deduction amounts) must equal the total Net rent present in all Rental property schedules",
                    //Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432117"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT89", Value = GetValueOrEmpty(report.TRT89) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT90", Value = GetValueOrEmpty(report.TRT90) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT91", Value = GetValueOrEmpty(report.TRT91) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT92", Value = GetValueOrEmpty(report.TRT92) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432117

        #region VR.ATO.TRT.432120

        /*  VR.ATO.TRT.432120
                    If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property

                Legacy Rule Format:
                    IF ([TRT89] <> NULL OR [TRT90] > 0 OR [TRT91] > 0 OR [TRT92] > 0) AND (COUNT(SCHEDULE = "RS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
                    [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
                    [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
                    [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (^TRT89 <> NULL OR ^TRT90 > 0 OR ^TRT91 > 0 OR ^TRT92 > 0) AND (CountDocument('RS') = 0)

        Data Elements:
    
            ^TRT89 = RP.JAUS:Income.Operating.RentalIncomeGross.Amount
    
            ^TRT90 = RP.JAUS:Expense.Interest.Amount
    
            ^TRT91 = RP.JAUS:Expense.CapitalWorksDeduction.Amount
    
            ^TRT92 = RP.JAUS:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
        protected void VRATOTRT432120(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432120
            
            assertion = false; // = ((report.TRT89 != null || report.TRT90.GetValueOrDefault() > 0 || report.TRT91.GetValueOrDefault() > 0 || report.TRT92.GetValueOrDefault() > 0) && @"RS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430081", Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental schedule must be present",
                    LongDescription = @"If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property",
                    //Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432120"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT89", Value = GetValueOrEmpty(report.TRT89) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT90", Value = GetValueOrEmpty(report.TRT90) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT91", Value = GetValueOrEmpty(report.TRT91) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT92", Value = GetValueOrEmpty(report.TRT92) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432120

        #region VR.ATO.TRT.432122

        /*  VR.ATO.TRT.432122
                    Total interest of schedules AND interest value in return must be equal

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND SUM([RS48]) <> [TRT90] 
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS48] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LoanInterest.Amount
                    [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND Sum(^RS48) <> ^TRT90

        Data Elements:
    
            ^TRT90 = RP.JAUS:Expense.Interest.Amount
    
            ^RS48 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LoanInterest.Amount
            */
        protected void VRATOTRT432122(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432122
            
            assertion = false; // = (@"RS") > 0 && ( == null ? 0 : .Sum(f => f.)) != report.TRT90.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430086", Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental Interest deductions must equal the total Interest deductions in the Rental Schedules",
                    LongDescription = @"The Interest deductions amount in the main form must equal the total of all Interest deductions in the Rental property schedules",
                    //Location = "/xbrli:xbrl/tns:Expense.Interest.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432122"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT90", Value = GetValueOrEmpty(report.TRT90) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432122

        #region VR.ATO.TRT.432124

        /*  VR.ATO.TRT.432124
                    Total of capital works deductions claimed in attached schedules are not equal to the total of capital works deductions claimed in the return.

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "RS") > 0) AND SUM([RS54]) <> [TRT91]
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS54] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
                    [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND Sum(^RS54) <> ^TRT91

        Data Elements:
    
            ^TRT91 = RP.JAUS:Expense.CapitalWorksDeduction.Amount
    
            ^RS54 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CapitalWorksDeduction.Amount
            */
        protected void VRATOTRT432124(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432124
            
            assertion = false; // = (@"RS") > 0 && ( == null ? 0 : .Sum(f => f.)) != report.TRT91.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430088", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital works deductions must equal the total Capital works deductions in the Rental Schedules",
                    LongDescription = @"The Capital works deductions amount in the main form must equal the total of all Capital works deductions in the Rental property schedules",
                    //Location = "/xbrli:xbrl/tns:Expense.CapitalWorksDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432124"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT91", Value = GetValueOrEmpty(report.TRT91) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432124

        #region VR.ATO.TRT.432126

        /*  VR.ATO.TRT.432126
                    Total of rental expenses less Interest AND Capital works (Special building write-off) of schedules AND value of Other rental deductions in return must be equal

                Legacy Rule Format:
                    IF  (COUNT(SCHEDULE = "RS") > 0) AND (SUM(ALL OCCURRENCES OF([RS40] + [RS41] + [RS42] + [RS43] + [RS44] + [RS45] + [RS46] + [RS47] + [RS49] + [RS50] + [RS51] + [RS52] + [RS53] + [RS55] + [RS56] + [RS57] + [RS58])) <> [TRT92])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [RS40] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Marketing.Amount
                    [RS41] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.BodyCorporate.Amount
                    [RS42] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Borrowing.Amount
                    [RS43] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Cleaning.Amount
                    [RS44] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CouncilRates.Amount
                    [RS45] = RS:RP:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
                    [RS46] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Gardening.Amount
                    [RS47] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Insurance.Amount
                    [RS49] = RS:RP:bafpr1.02.00:Expense.LandTax.Amount
                    [RS50] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LegalFees.Amount
                    [RS51] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.PestControl.Amount
                    [RS52] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.AgentFeesCommission.Amount
                    [RS53] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
                    [RS55] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.OfficeSupplies.Amount
                    [RS56] = RS:RP:bafpr1.02.00:Expense.Travel.Amount
                    [RS57] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Water.Amount
                    [RS58] = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Sundry.Amount
                    [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
                    (CountDocument('RS') > 0) AND (Sum((^RS40 + ^RS41 + ^RS42 + ^RS43 + ^RS44 + ^RS45 + ^RS46 + ^RS47 + ^RS49 + ^RS50 + ^RS51 + ^RS52 + ^RS53 + ^RS55 + ^RS56 + ^RS57 + ^RS58)) <> ^TRT92)

        Data Elements:
    
            ^TRT92 = RP.JAUS:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
    
            ^RS40 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Marketing.Amount
    
            ^RS41 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.BodyCorporate.Amount
    
            ^RS42 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Borrowing.Amount
    
            ^RS43 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Cleaning.Amount
    
            ^RS44 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.CouncilRates.Amount
    
            ^RS45 = RS:RP:bafpr1.02.04:Expense.DepreciationandAmortisation.Amount
    
            ^RS46 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Gardening.Amount
    
            ^RS47 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Insurance.Amount
    
            ^RS49 = RS:RP:bafpr1.02.00:Expense.LandTax.Amount
    
            ^RS50 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.LegalFees.Amount
    
            ^RS51 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.PestControl.Amount
    
            ^RS52 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.AgentFeesCommission.Amount
    
            ^RS53 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.RepairsAndMaintenance.Amount
    
            ^RS55 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.OfficeSupplies.Amount
    
            ^RS56 = RS:RP:bafpr1.02.00:Expense.Travel.Amount
    
            ^RS57 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Water.Amount
    
            ^RS58 = RS:RP:bafpr2.02.04:Expense.RealEstateProperty.Sundry.Amount
            */
        protected void VRATOTRT432126(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432126
            
            assertion = false; // = (@"RS") > 0 &&  +  +  +  +  +  +  +  +  +  +  +  +  +  +  +  + ) != report.TRT92.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430090", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other rental deductions must equal the total other deductions in the Rental Schedules",
                    LongDescription = @"The Other rental deductions amount in the main form must equal the total of all rental expenses in the Rental property, excluding Interest deductions and Capital works deductions",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432126"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT92", Value = GetValueOrEmpty(report.TRT92) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432126

        #region VR.ATO.TRT.432189

        /*  VR.ATO.TRT.432189
                    The Net Capital gain amount on the main return form must equal the Total net capital gain amount on the Capital Gains Tax schedule.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([TRT133] <> [CGTS131])
                      RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS131] = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND (^TRT133 <> ^CGTS131)

        Data Elements:
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^CGTS131 = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            */
        protected void VRATOTRT432189(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432189
            
            assertion = false; // = (@"CGTS") == 1 && report.TRT133.GetValueOrDefault() != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434036", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gains in return must equal total net capital gain in CGT schedule.",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432189"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432189

        #region VR.ATO.TRT.432208

        /*  VR.ATO.TRT.432208
                    Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities. Trusts are required to indicate whether continuity of majority ownership test passed for each year of loss.

                Legacy Rule Format:
                    IF [TRT32] = "179" AND [TRT144] > 100000 AND (COUNT(SCHEDULE = "LS") = 1) AND [LS15] = NULL AND [LS16] = NULL AND [LS17] = NULL AND [LS18] = NULL AND [LS19] = NULL AND [LS53] = NULL 
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [LS15] = LS:RP.Y0:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS16] = LS:RP.Y0-1:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS17] = LS:RP.Y0-2:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS18] = LS:RP.Y0-3:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS19] = LS:RP.Y0-4:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [LS53] = LS:RP.Y0-5P:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
                    [TRT32] = TRT:RP:pyde.02.26:OrganisationDetails.OrganisationType.Code
                    [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount

            Technical Business Rule Format:
                    ^TRT32 = '179' AND ^TRT144 > 100000 AND (CountDocument('LS') = 1) AND ^LS15 = NULL AND ^LS16 = NULL AND ^LS17 = NULL AND ^LS18 = NULL AND ^LS19 = NULL AND ^LS53 = NULL

        Data Elements:
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^LS15 = LS:RP.Y0:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
            ^LS16 = LS:RP.Y0-1:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
            ^LS17 = LS:RP.Y0-2:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
            ^LS18 = LS:RP.Y0-3:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
            ^LS19 = LS:RP.Y0-4:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
            ^LS53 = LS:RP.Y0-5P:rvctc3.02.00:Tax.Losses.CarriedForward.ContinuityOfMajorityOwnershipTestPassed.Indicator
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
            */
        protected void VRATOTRT432208(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432208
            
            assertion = false; // = (report.TRT32 == @"179" && report.TRT144.GetValueOrDefault() > 100000 && @"LS") == 1 &&  == null &&  == null &&  == null &&  == null &&  == null &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432208", Severity = ProcessMessageSeverity.Error,
                    Description = @"Widely held trusts must indicate whether continuity of majority ownership test passed for each year of loss",
                    LongDescription = @"Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxLossesDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432208"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432208

        #region VR.ATO.TRT.432210

        /*  VR.ATO.TRT.432210
                    Where the total of tax losses AND net capital losses exceeds $100000, a Losses schedule must be attached.

                Legacy Rule Format:
                    IF ([TRT145] + [TRT475]) > 100000 AND [TRT44] <> TRUE AND (COUNT(SCHEDULE = "LS") = 0)
                         RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT44] = TRT:RP:pyde.02.00:OrganisationDetails.TaxConsolidationStatus.Indicator
                    [TRT145] = TRT:RP.Closing:rvctc3.02.02:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
                    [TRT475] = TRT:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (^TRT145 + ^TRT475) > 100000 AND ^TRT44 <> TRUE AND (CountDocument('LS') = 0)

        Data Elements:
    
            ^TRT145 = RP.Closing:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
    
            ^TRT44 = RP:OrganisationDetails.TaxConsolidationStatus.Indicator
    
            ^TRT475 = RP.Closing(Instant):Capital.Losses.CarriedForward.Net.Amount
            */
        protected void VRATOTRT432210(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432210
            
            assertion = false; // = (report.TRT145.GetValueOrDefault() + report.TRT475.GetValueOrDefault() > 100000 && report.TRT44 != true && @"LS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432210", Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule must be attached where total of tax losses and net capital losses exceeds $100000",
                    //Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432210"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT145", Value = GetValueOrEmpty(report.TRT145) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT475", Value = GetValueOrEmpty(report.TRT475) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT44", Value = GetValueOrEmpty(report.TRT44) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432210

        #region VR.ATO.TRT.432212

        /*  VR.ATO.TRT.432212
                    Total of the tax losses carried forward to later income years in the losses schedule for unconsolidated entities must equal tax losses carried forward to later income years on the main income tax return.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND [LS50] <> [TRT145] 
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [LS50] = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
                    [TRT145] = TRT:RP.Closing:rvctc3.02.02:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND ^LS50 <> ^TRT145

        Data Elements:
    
            ^TRT145 = RP.Closing:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
    
            ^LS50 = LS:RP.ALL:rvctc3.02.00:Tax.Losses.CarriedForward.Total.Amount
            */
        protected void VRATOTRT432212(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432212
            
            assertion = false; // = (@"LS") == 1 &&  != report.TRT145.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434086", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax losses carried forward to later income years in schedule must equal amount in main form",
                    LongDescription = @"The total of the 'Tax losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Tax losses carried forward to later income years' on the main form",
                    //Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432212"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT145", Value = GetValueOrEmpty(report.TRT145) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432212

        #region VR.ATO.TRT.432213

        /*  VR.ATO.TRT.432213
                    Unapplied net capital losses carried forward on the Capital Gains Tax schedule (if lodged) must equal Net capital losses carried forward on the main income tax return.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([CGTS124] + [CGTS125]) <> [TRT475]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS124] = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [CGTS125] = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [TRT475] = TRT:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND (^CGTS124 + ^CGTS125) <> ^TRT475

        Data Elements:
    
            ^TRT475 = RP.Closing(Instant):Capital.Losses.CarriedForward.Net.Amount
    
            ^CGTS124 = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
    
            ^CGTS125 = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            */
        protected void VRATOTRT432213(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432213
            
            assertion = false; // = (@"CGTS") == 1 &&  +  != report.TRT475.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434088", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in CGT schedule",
                    LongDescription = @"The total of the 'Unapplied net capital losses carried forward'  in the Capital Gains Tax schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432213"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT475", Value = GetValueOrEmpty(report.TRT475) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432213

        #region VR.ATO.TRT.432215

        /*  VR.ATO.TRT.432215
                    Net capital losses carried forward to later income years - Total in the Losses schedule for unconsolidated entities is equal to the Net capital losses carried forward to later income years on the main income tax return.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "LS") = 1) AND [LS52] <> [TRT475]  
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [LS52] = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [TRT475] = TRT:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('LS') = 1) AND ^LS52 <> ^TRT475

        Data Elements:
    
            ^TRT475 = RP.Closing(Instant):Capital.Losses.CarriedForward.Net.Amount
    
            ^LS52 = LS:RP.ALL(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            */
        protected void VRATOTRT432215(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432215
            
            assertion = false; // = (@"LS") == 1 &&  != report.TRT475.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434087", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in Losses schedule",
                    LongDescription = @"The total of the 'Net capital losses carried forward to later income years'  for unconsolidated entities in the Losses schedule must equal 'Net capital losses carried forward to later income years' on the main form",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432215"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT475", Value = GetValueOrEmpty(report.TRT475) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432215

        #region VR.ATO.TRT.432429

        /*  VR.ATO.TRT.432429
                    Losses schedule must be attached

                Legacy Rule Format:
                    IF [TRT32] = "179" AND [TRT144] > 100000 AND (COUNT(SCHEDULE = "LS") = 0) 
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT32] = TRT:RP:pyde.02.26:OrganisationDetails.OrganisationType.Code
                    [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount

            Technical Business Rule Format:
                    ^TRT32 = '179' AND ^TRT144 > 100000 AND (CountDocument('LS') = 0)

        Data Elements:
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
            */
        protected void VRATOTRT432429(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432429
            
            assertion = false; // = (report.TRT32 == @"179" && report.TRT144.GetValueOrDefault() > 100000 && @"LS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432429", Severity = ProcessMessageSeverity.Error,
                    Description = @"Losses schedule must be attached",
                    LongDescription = @"Public unit trust (listed other than cash management unit trust) that have Tax losses deducted of greater than $100,000 must complete Part B1 of the Losses schedule for unconsolidated entities",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxLossesDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432429"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432429

        #region VR.ATO.TRT.432646

        /*  VR.ATO.TRT.432646
                    The sum of the amounts showing at attributable income of controlled foreign companies and controlled foreign trusts on IDS should be equal to or less than the sum of the amounts at Listed country and Unlisted country on the main return form.

                Legacy Rule Format:
                    IF (([IDS191] + [IDS192] + [IDS193]) > ([TRT134] + [TRT136]))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS191] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
                    [IDS192] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
                    [IDS193] = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
                    [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    ((^IDS191 + ^IDS192 + ^IDS193) > (^TRT134 + ^TRT136))

        Data Elements:
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^IDS191 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount
    
            ^IDS192 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount
    
            ^IDS193 = IDS:RP:bafpr1.02.12:Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
            */
        protected void VRATOTRT432646(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432646
            
            assertion = false; // = ( +  +  > report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432714", Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income incorrect",
                    LongDescription = @"The sum of amounts at attributable income of controlled foreign companies and controlled foreign trusts on the IDS should be equal to or less than the sum of amounts at Listed country and Unlisted country on the trust return form.",
                    //Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCUCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432646"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432646

        #region VR.ATO.TRT.432647

        /*  VR.ATO.TRT.432647
                    The amount showing at section 456 –attributable income controlled foreign companies (question 22 Label D) on IDS should be equal to or less than the amount at Listed country on the main return form (company item 8 label B; partnership and trust item 22 label M).

                Legacy Rule Format:
                    IF ([TRT134] > 0) AND ([IDS188] > [TRT134])
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS188] = IDS:RP.CFCLC:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncome.Amount
                    [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    (^TRT134 > 0) AND (^IDS188 > ^TRT134)

        Data Elements:
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^IDS188 = IDS:RP.CFCLC:bafpr1.02.12:Income.ControlledForeignCompaniesAttributableIncome.Amount
            */
        protected void VRATOTRT432647(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432647
            
            assertion = false; // = (report.TRT134.GetValueOrDefault() > 0 &&  > report.TRT134.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432647", Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income incorrect",
                    LongDescription = @"Assessable income of controlled foreign companies and controlled foreign trusts under section 456 for Listed countries (Question 22 Label A) on the IDS should be equal to or less than Attributed foreign income - Listed country on the trust return.",
                    //Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432647"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432647

        #region VR.ATO.TRT.432648

        /*  VR.ATO.TRT.432648
                    Listed country, Unlisted country or Transferor trust (Item 8 Labels B, U or V) has been completed on the trust return form and the number of controlled foreign companies or trusts (Question 21) on the IDS has not been shown. Show zero at Question 21 Labels B, C or D on the IDS if at the end of your income year you no longer held an interest in CFCs or CFTs.

                Legacy Rule Format:
                    IF ([TRT134] > 0 OR [TRT136] > 0) AND ([IDS185] = NULL AND [IDS186] = NULL AND [IDS187] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS185] = IDS:RP.CFCLC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    [IDS186] = IDS:RP.CFCSP:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    [IDS187] = IDS:RP.CFCUC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
                    [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
                    (^TRT134 > 0 OR ^TRT136 > 0) AND (^IDS185 = NULL AND ^IDS186 = NULL AND ^IDS187 = NULL)

        Data Elements:
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^IDS185 = IDS:RP.CFCLC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
    
            ^IDS186 = IDS:RP.CFCSP:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
    
            ^IDS187 = IDS:RP.CFCUC:bafot.02.29:Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
            */
        protected void VRATOTRT432648(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432648
            
            assertion = false; // = ((report.TRT134.GetValueOrDefault() > 0 || report.TRT136.GetValueOrDefault() > 0) && ( == null &&  == null &&  == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432756", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of controlled foreign companies or trusts required",
                    LongDescription = @"Listed country, Unlisted country or Transferor trust (Item 8 Labels B, U or V) has been completed on the trust return form and the number of controlled foreign companies or trusts (Question 21) on the IDS has not been shown. Show zero at Question 21 Labels B, C or D on the IDS if at the end of your income year you no longer held an interest in CFCs or CFTs.",
                    //Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432648"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432648

        #region VR.ATO.TRT.432649

        /*  VR.ATO.TRT.432649
                    If there is a yes response to question 4 label A on the IDS, there must also be a yes response to question 29 label C (transactions with specified countries) on the trust or partnership return form.

                Legacy Rule Format:
                    IF [TRT151] = FALSE AND [IDS9] = TRUE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS9] = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
                    [TRT151] = TRT:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator

            Technical Business Rule Format:
                    ^TRT151 = FALSE AND ^IDS9 = TRUE

        Data Elements:
    
            ^TRT151 = RP:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
    
            ^IDS9 = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator
            */
        protected void VRATOTRT432649(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432649
            
            assertion = false; // = (report.TRT151 == false &&  == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432649", Severity = ProcessMessageSeverity.Error,
                    Description = @"Transactions with specified countries question incorrect",
                    LongDescription = @"If there is a 'True' response at 'Did you have dealings with international related parties located in specified countries?' on the IDS then the response at Overseas transactions - Transactions with specified countries on the trust return must be 'True'.",
                    //Location = "/xbrli:xbrl/tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432649"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT151", Value = GetValueOrEmpty(report.TRT151) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432649

        #region VR.ATO.TRT.432650

        /*  VR.ATO.TRT.432650
                    An amount is showing at debt deductions in earning non-assessable foreign income claimed under s25-90 (question 25 Label B on IDS) and there is no amount at Interest expenses, Other deductions or Interest expenses overseas on the partnership or trust return form (item 5 label I, item 18 label Q, item 54 label Q)

                Legacy Rule Format:
                    IF ([TRT470] = NULL OR [TRT470] = 0 ) AND ([TRT182] = NULL OR [TRT182] = 0 ) AND ([TRT129] = NULL OR [TRT129] = 0) AND ([IDS207] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS207] = IDS:RP:rvctc2.02.09:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount
                    [TRT129] = TRT:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(trt.0008.lodge.req.02.00:OtherDeductions)
                    [TRT182] = TRT:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount
                    [TRT470] = TRT:RP.JAUS:bafpr2.02.17:Expense.Operating.Interest.Amount

            Technical Business Rule Format:
                    (^TRT470 = NULL OR ^TRT470 = 0) AND (^TRT182 = NULL OR ^TRT182 = 0) AND (^TRT129 = NULL OR ^TRT129 = 0) AND (^IDS207 > 0)

        Data Elements:
    
            ^TRT470 = RP.JAUS:Expense.Operating.Interest.Amount
    
            ^IDS207 = IDS:RP:rvctc2.02.09:IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount
    
            ^TRT129 = RP.JAUS:OtherDeductions:Expense.DeductibleOther.Amount
    
            ^TRT182 = RP.JFOREIGN:Expense.Interest.Amount
            */
        protected void VRATOTRT432650(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432650
            
            assertion = false; // = ((report.TRT470 == null || report.TRT470 == 0) && (report.TRT182 == null || report.TRT182 == 0) && (otherDeductions.TRT129 == null || otherDeductions.TRT129 == 0) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432650", Severity = ProcessMessageSeverity.Error,
                    Description = @"Interest expenses required",
                    LongDescription = @"If Debt deductions in earning non-assessable foreign income claimed under s25-90 has been completed on the IDS then Total interest expenses or Other deductions or Interest expenses overseas must be greater than zero on trust return form.",
                    //Location = "/xbrli:xbrl/tns:Expense.Operating.Interest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432650"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT470", Value = GetValueOrEmpty(report.TRT470) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT182", Value = GetValueOrEmpty(report.TRT182) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT129", Value = GetValueOrEmpty(otherDeductions.TRT129) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432650

        #region VR.ATO.TRT.432651

        /*  VR.ATO.TRT.432651
                    An amount is showing at Total amount of your share of net income/distribution of profit (question 29 Label C on IDS) and there is no amount at Item 23 other assessable foreign source income, Label B Gross on the partnership or trust return form.

                Legacy Rule Format:
                    IF ([TRT140] = NULL) AND ([IDS219] > 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS219] = IDS:RP:bafpo7.02.11:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount
                    [TRT140] = TRT:RP.JFOREIGN:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
                    (^TRT140 = NULL) AND (^IDS219 > 0)

        Data Elements:
    
            ^TRT140 = RP.JFOREIGN:Income.Other.Amount
    
            ^IDS219 = IDS:RP:bafpo7.02.11:Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount
            */
        protected void VRATOTRT432651(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432651
            
            assertion = false; // = (report.TRT140 == null &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432651", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross other assessable foreign source income required",
                    LongDescription = @"If Total amount of your share of net income/distribution of profit (Question 29 Label C) has been completed on the IDS then the Gross Other assessable foreign source income (Item 23 Label B) must be completed on the trust return form.",
                    //Location = "/xbrli:xbrl/tns:Income.Other.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432651"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT140", Value = GetValueOrEmpty(report.TRT140) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432651

        #region VR.ATO.TRT.432652

        /*  VR.ATO.TRT.432652
                    Amounts are showing at question 26 on IDS but there is no 'yes' response to the question concerning whether you have a CGT event during the year (Item 7 label G on the company return form; item 21 label G on the partnership and trust return forms).

                Legacy Rule Format:
                    IF ([TRT131] = FALSE) AND ([IDS208] = TRUE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS208] = IDS:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator
                    [TRT131] = TRT:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator

            Technical Business Rule Format:
                    (^TRT131 = FALSE) AND (^IDS208 = TRUE)

        Data Elements:
    
            ^TRT131 = RP:CapitalGainsTax.Event.Indicator
    
            ^IDS208 = IDS:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator
            */
        protected void VRATOTRT432652(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432652
            
            assertion = false; // = (report.TRT131 == false &&  == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432652", Severity = ProcessMessageSeverity.Error,
                    Description = @"Check response to CGT event question",
                    LongDescription = @"If there is a 'True' response to 'Did you have any capital gains tax (CGT) events in relation to your interest in a foreign company?' On the IDS then a 'True' response is required at 'Did you have a CGT event during the year?' on the trust return.",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.Event.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432652"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT131", Value = GetValueOrEmpty(report.TRT131) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432652

        #region VR.ATO.TRT.432653

        /*  VR.ATO.TRT.432653
                    An amount is showing at question 26 on IDS and there is no amount at net capital gain (Item 7 label A on the company return form, Item 21 label A on the partnership and trust return forms).

                Legacy Rule Format:
                    IF ([TRT133] = NULL AND ([IDS209] > 0))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS209] = IDS:RP:rvctc3.02.10:CapitalGainsTax.ActiveForeignBusinessCapitalGains.Amount
                    [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    (^TRT133 = NULL AND (^IDS209 > 0))

        Data Elements:
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^IDS209 = IDS:RP:rvctc3.02.10:CapitalGainsTax.ActiveForeignBusinessCapitalGains.Amount
            */
        protected void VRATOTRT432653(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432653
            
            assertion = false; // = (report.TRT133 == null &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432653", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gain required",
                    LongDescription = @"If Capital gain amounts has been completed on the IDS then Net capital gain must be completed on the trust return.",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432653"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432653

        #region VR.ATO.TRT.432655

        /*  VR.ATO.TRT.432655
                    If there is an amount at question 6a, label C on the IDS it must be less than or equal to the amount at item 55 label R on the partnership and trust return form.

                Legacy Rule Format:
                    IF ([TRT183] = 0 OR [TRT183] = NULL) AND ([IDS22] > 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^TRT183 = 0 OR ^TRT183 = NULL) AND (^IDS22 > 0)

        Data Elements:
    
            ^TRT183 = RP.JFOREIGN:Expense.Royalties.Amount
    
            ^IDS22 = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
            */
        protected void VRATOTRT432655(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432655
            
            assertion = false; // = ((report.TRT183 == 0 || report.TRT183 == null) &&  > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432655", Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalty expenses overseas required",
                    LongDescription = @"If a Royalties deduction amount (Question 6a Label C) has been completed on the IDS then Royalty expenses overseas (BPI Label R) must be greater than zero on the partnership or trust return form.",
                    //Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432655"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT183", Value = GetValueOrEmpty(report.TRT183) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432655

        #region VR.ATO.TRT.432656

        /*  VR.ATO.TRT.432656
                    Where there is a Royalties deduction amount present on the IDS, there must also be an amount at Royalty expenses overseas on the partnership or trust return form.

                Legacy Rule Format:
                    IF ([TRT183] > 0) AND ([TRT183] < [IDS22])
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^TRT183 > 0) AND (^TRT183 < ^IDS22)

        Data Elements:
    
            ^TRT183 = RP.JFOREIGN:Expense.Royalties.Amount
    
            ^IDS22 = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
            */
        protected void VRATOTRT432656(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432656
            
            assertion = false; // = (report.TRT183.GetValueOrDefault() > 0 && report.TRT183.GetValueOrDefault() < );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432656", Severity = ProcessMessageSeverity.Error,
                    Description = @"Royalties deductions incorrect",
                    LongDescription = @"If Royalties deductions has been completed on the IDS then it must be less than or equal to Royalty expenses overseas on the trust return form",
                    //Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432656"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT183", Value = GetValueOrEmpty(report.TRT183) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432656

        #region VR.ATO.TRT.432675

        /*  VR.ATO.TRT.432675
                    Ensure an ‘International dealings schedule’ (IDS) is completed when a value greater than zero is present at label Interest expenses overseas

                Legacy Rule Format:
                    IF ([TRT182] > 0) AND (COUNT(SCHEDULE = "IDS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT182] = TRT:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
                    (^TRT182 > 0) AND (CountDocument('IDS') = 0)

        Data Elements:
    
            ^TRT182 = RP.JFOREIGN:Expense.Interest.Amount
            */
        protected void VRATOTRT432675(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432675
            
            assertion = false; // = (report.TRT182.GetValueOrDefault() > 0 && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432675", Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"An 'International dealings schedule' (IDS) is not attached when a value greater than zero is present at label Interest expenses overseas",
                    //Location = "/xbrli:xbrl/tns:Expense.Interest.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432675"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT182", Value = GetValueOrEmpty(report.TRT182) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432675

        #region VR.ATO.TRT.432676

        /*  VR.ATO.TRT.432676
                    Ensure an ‘International dealings schedule’ (IDS) is completed when a value greater than zero is present at label Royalty expenses overseas

                Legacy Rule Format:
                    IF ([TRT183] > 0) AND (COUNT(SCHEDULE = "IDS") = 0)
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
                    (^TRT183 > 0) AND (CountDocument('IDS') = 0)

        Data Elements:
    
            ^TRT183 = RP.JFOREIGN:Expense.Royalties.Amount
            */
        protected void VRATOTRT432676(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432676
            
            assertion = false; // = (report.TRT183.GetValueOrDefault() > 0 && @"IDS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432676", Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule not attached",
                    LongDescription = @"An 'International dealings schedule' (IDS) is not attached when a value greater than zero is present at label Royalty expenses overseas",
                    //Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432676"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT183", Value = GetValueOrEmpty(report.TRT183) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432676

        #region VR.ATO.TRT.432707

        /*  VR.ATO.TRT.432707
                    If there is a 'NO' response to the questions concerning whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust and whether the value of dealings with international related parties is greater than $2 million and whether the thin capitalisation provisions apply, and no amounts have been completed at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached.

                Legacy Rule Format:
                    IF ([TRT138] = FALSE AND [TRT148] = FALSE AND [TRT449] = FALSE) AND (([TRT182] = NULL 
                    OR [TRT182] = 0) AND ([TRT183] = NULL OR [TRT183] = 0)) AND (COUNT (Schedule = “IDS”) = 1)
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT138] = TRT:RP.JFOREIGN:bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
                    [TRT148] = TRT:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
                    [TRT182] = TRT:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount
                    [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount
                    [TRT449] = TRT:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator

            Technical Business Rule Format:
                    (^TRT138 = FALSE AND ^TRT148 = FALSE AND ^TRT449 = FALSE) AND ((^TRT182 = NULL OR ^TRT182 = 0) AND (^TRT183 = NULL OR ^TRT183 = 0)) AND (CountDocument('IDS') = 1)

        Data Elements:
    
            ^TRT138 = RP.JFOREIGN:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
    
            ^TRT148 = RP:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
    
            ^TRT182 = RP.JFOREIGN:Expense.Interest.Amount
    
            ^TRT183 = RP.JFOREIGN:Expense.Royalties.Amount
    
            ^TRT449 = RP:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
            */
        protected void VRATOTRT432707(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432707
            
            assertion = false; // = (report.TRT138 == false && report.TRT148 == false && report.TRT449 == false && ((report.TRT182 == null || report.TRT182 == 0) && (report.TRT183 == null || report.TRT183 == 0)) && @"IDS") == 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438031", Severity = ProcessMessageSeverity.Error,
                    Description = @"International Dealings Schedule is not required",
                    LongDescription = @"If there is a 'NO' response to the questions concerning whether the entity had either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust and whether the value of dealings with international related parties is greater than $2 million and whether the thin capitalisation provisions apply, and no amounts have been completed at Interest expenses overseas and Royalty expenses overseas, an International Dealings schedule must not be attached.",
                    //Location = "/xbrli:xbrl/tns:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432707"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT138", Value = GetValueOrEmpty(report.TRT138) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT148", Value = GetValueOrEmpty(report.TRT148) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT449", Value = GetValueOrEmpty(report.TRT449) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT182", Value = GetValueOrEmpty(report.TRT182) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT183", Value = GetValueOrEmpty(report.TRT183) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432707

        #region VR.ATO.TRT.432720

        /*  VR.ATO.TRT.432720
                    If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS

                Legacy Rule Format:
                    IF ([TRT356] > 0 OR [TRT357] > 0) 
                    AND (COUNT(SCHEDULE = "IDS") = 1 AND [IDS177] <> TRUE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS177] = IDS:RP:rvctc4.02.05:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator
                    [TRT356] = TRT:RP.TOFA:bafpr1.02.04:Income.GainsTotal.Amount
                    [TRT357] = TRT:RP.TOFA:bafpr1.02.04:Expense.LossesTotal.Amount

            Technical Business Rule Format:
                    (^TRT356 > 0 OR ^TRT357 > 0) AND (CountDocument('IDS') = 1 AND ^IDS177 <> TRUE)

        Data Elements:
    
            ^TRT356 = RP.TOFA:Income.GainsTotal.Amount
    
            ^IDS177 = IDS:RP:rvctc4.02.05:InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator
    
            ^TRT357 = RP.TOFA:Expense.LossesTotal.Amount
            */
        protected void VRATOTRT432720(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432720
            
            assertion = false; // = ((report.TRT356.GetValueOrDefault() > 0 || report.TRT357.GetValueOrDefault() > 0) && (@"IDS") == 1 &&  != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432762", Severity = ProcessMessageSeverity.Error,
                    Description = @"Response to TOFA gains or losses is required",
                    LongDescription = @"If there is an amount at either 'Total TOFA gains' or 'Total TOFA Losses' that indicates you made a gain or loss from a financial arrangement subject to the TOFA rules, then there must be a 'True' response to 'Are you subject to the Taxation of Financial Arrangements rules contained in Division 230 of the ITAA 1997?' on the IDS.",
                    //Location = "/xbrli:xbrl/tns:Income.GainsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432720"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT356", Value = GetValueOrEmpty(report.TRT356) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT357", Value = GetValueOrEmpty(report.TRT357) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432720

        #region VR.ATO.TRT.432797

        /*  VR.ATO.TRT.432797
                    If the entity has self-assessed as a Significant Global Entity (SGE), then 'Are you a small business entity, not a significant global entity and your international related party dealings do not exceed 50% of your current year aggregated turnover?' must not be 'TRUE' on the International Dealings Schedule (IDS) form.

                Legacy Rule Format:
                    IF [TRT487] = TRUE AND (COUNT(SCHEDULE = "IDS") = 1 AND [IDS425] = TRUE)
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS425] = IDS:RP:rvctc4.02.17:InternationalDealings.SmallBusinessConcessionsEligibility.Indicator
                    [TRT487] = TRT:RP:pyde.02.43:OrganisationDetails.SignificantGlobalEntityStatus.Indicator

            Technical Business Rule Format:
                    ^TRT487 = TRUE AND (CountDocument('IDS') = 1 AND ^IDS425 = TRUE)

        Data Elements:
    
            ^TRT487 = RP:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
    
            ^IDS425 = IDS:RP:rvctc4.02.17:InternationalDealings.SmallBusinessConcessionsEligibility.Indicator
            */
        protected void VRATOTRT432797(List<ProcessMessageDocument> response, TRT2020 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TRT.432797
            
            assertion = false; // = (report.TRT487 == true && (@"IDS") == 1 &&  == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432797", Severity = ProcessMessageSeverity.Error,
                    Description = @"Significant Global Entity question on the International Dealings Schedule (IDS) form must not be 'TRUE'.",
                    //Location = "/xbrli:xbrl/tns:OrganisationDetails.SignificantGlobalEntityStatus.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432797"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT487", Value = GetValueOrEmpty(report.TRT487) });
    
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.TRT.432797
    #endregion Cross Form Rules - not supported yet
        #region ValidateCrossFormRules

        public List<ProcessMessageDocument> ValidateCrossFormRules(TRT2020 report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            #region Cross Form Rules
            VRATOGEN430008(response, report);
            VRATOGEN438002(response, report);
            VRATOGEN438017(response, report);
            VRATOGEN438022(response, report);
            VRATOGEN438023(response, report);
            VRATOGEN438024(response, report);
            VRATOGEN438025(response, report);
            VRATOGEN438026(response, report);
            VRATOGEN438027(response, report);
            VRATOGEN438028(response, report);
            VRATOTRT432011(response, report);
            VRATOTRT432017(response, report);
            VRATOTRT432019(response, report);
            VRATOTRT432020(response, report);
            VRATOTRT432042(response, report);
            VRATOTRT432043(response, report);
            VRATOTRT432044(response, report);
            VRATOTRT432051(response, report);
            VRATOTRT432088(response, report);
            VRATOTRT432115(response, report);
            VRATOTRT432116(response, report);
            VRATOTRT432117(response, report);
            VRATOTRT432120(response, report);
            VRATOTRT432122(response, report);
            VRATOTRT432124(response, report);
            VRATOTRT432126(response, report);
            VRATOTRT432189(response, report);
            VRATOTRT432208(response, report);
            VRATOTRT432210(response, report);
            VRATOTRT432212(response, report);
            VRATOTRT432213(response, report);
            VRATOTRT432215(response, report);
            VRATOTRT432429(response, report);
            VRATOTRT432646(response, report);
            VRATOTRT432647(response, report);
            VRATOTRT432648(response, report);
            VRATOTRT432649(response, report);
            VRATOTRT432650(response, report);
            VRATOTRT432651(response, report);
            VRATOTRT432652(response, report);
            VRATOTRT432653(response, report);
            VRATOTRT432655(response, report);
            VRATOTRT432656(response, report);
            VRATOTRT432675(response, report);
            VRATOTRT432676(response, report);
            VRATOTRT432707(response, report);
            VRATOTRT432720(response, report);
            VRATOTRT432797(response, report);
            #endregion Cross Form Rules
            return response;
        }

        #endregion
    }
}
